@@ -10,6 +10,7 @@ from eventlet.event import Event
 from kombu import Connection, Exchange, Queue
 from kombu.pools import producers
 
+from nameko.constants import DEFAULT_RETRY_POLICY
 from nameko.exceptions import (
     MethodNotFound, UnknownService, UnserializableValueError,
     serialize, deserialize)
@@ -133,7 +134,6 @@ def rpc_consumer():
 # pylint: disable=E1101,E1123
 class RpcProvider(EntrypointProvider, HeaderDecoder):
 
-    _shadow = None
     rpc_consumer = rpc_consumer(shared=CONTAINER_SHARED)
 
     def prepare(self):
@@ -143,26 +143,14 @@ class RpcProvider(EntrypointProvider, HeaderDecoder):
         self.rpc_consumer.unregister_provider(self)
         super(RpcProvider, self).stop()
 
-    def bind(self, name, container):
-        super(RpcProvider, self).bind(name, container)
-
-        # Generate a 'shadow' lambda that can be used to verify
-        # invocation args against our method signature without running the
-        # actual entrypoint - see :meth:`RpcProvider.check_signature`.
-        fn = getattr(self.container.service_cls, self.name)
-        argspec = inspect.getargspec(fn)
-        argspec.args[:1] = []  # remove self
-        signature = inspect.formatargspec(*argspec)[1:-1]  # remove parens
-
-        src = "lambda {}: None".format(signature)
-        self.shadow = eval(src)
-
     def check_signature(self, args, kwargs):
+        service_cls = self.container.service_cls
+        fn = getattr(service_cls, self.name)
         try:
-            self.shadow(*args, **kwargs)
+            service_instance = None  # fn is unbound
+            inspect.getcallargs(fn, service_instance, *args, **kwargs)
         except TypeError as exc:
-            msg = str(exc).replace('<lambda>', self.name)
-            raise IncorrectSignature(msg)
+            raise IncorrectSignature(str(exc))
 
     def handle_message(self, body, message):
         args = body['args']
@@ -195,15 +183,10 @@ def rpc():
 
 class Responder(object):
 
-    def __init__(self, message, retry=True, retry_policy=None):
-        self._connection = None
+    def __init__(self, message):
         self.message = message
-        self.retry = retry
-        if retry_policy is None:
-            retry_policy = {'max_retries': 3}
-        self.retry_policy = retry_policy
 
-    def send_response(self, container, result, exc_info):
+    def send_response(self, container, result, exc_info, **kwargs):
 
         error = None
         if exc_info is not None:
@@ -226,6 +209,9 @@ class Responder(object):
 
         exchange = get_rpc_exchange(container)
 
+        retry = kwargs.pop('retry', True)
+        retry_policy = kwargs.pop('retry_policy', DEFAULT_RETRY_POLICY)
+
         with producers[conn].acquire(block=True) as producer:
 
             routing_key = self.message.properties['reply_to']
@@ -235,9 +221,9 @@ class Responder(object):
 
             _log.debug('publish response %s:%s', routing_key, correlation_id)
             producer.publish(
-                msg, retry=self.retry, retry_policy=self.retry_policy,
+                msg, retry=retry, retry_policy=retry_policy,
                 exchange=exchange, routing_key=routing_key,
-                correlation_id=correlation_id)
+                correlation_id=correlation_id, **kwargs)
 
         return result, exc_info
 
@@ -332,8 +318,7 @@ class MethodProxy(HeaderEncoder):
         self.reply_listener = reply_listener
 
     def __call__(self, *args, **kwargs):
-        _log.debug('invoking %s', self,
-                   extra=self.worker_ctx.extra_for_logging)
+        _log.debug('invoking %s', self)
 
         worker_ctx = self.worker_ctx
         container = worker_ctx.container
@@ -365,8 +350,6 @@ class MethodProxy(HeaderEncoder):
         exchange = get_rpc_exchange(container)
 
         with producers[conn].acquire(block=True) as producer:
-            # TODO: should we enable auto-retry,
-            #      should that be an option in __init__?
 
             headers = self.get_message_headers(worker_ctx)
             correlation_id = str(uuid.uuid4())
@@ -383,16 +366,16 @@ class MethodProxy(HeaderEncoder):
                 reply_to=reply_to_routing_key,
                 headers=headers,
                 correlation_id=correlation_id,
+                retry=True,
+                retry_policy=DEFAULT_RETRY_POLICY
             )
 
             if not producer.channel.returned_messages.empty():
                 raise UnknownService(self.service_name)
 
-        _log.debug('Waiting for RPC reply event %s', self,
-                   extra=worker_ctx.extra_for_logging)
+        _log.debug('Waiting for RPC reply event %s', self)
         resp_body = reply_event.wait()
-        _log.debug('RPC reply event complete %s %s', self, resp_body,
-                   extra=worker_ctx.extra_for_logging)
+        _log.debug('RPC reply event complete %s %s', self, resp_body)
 
         error = resp_body.get('error')
         if error:
